
/*****************************************************************************/
/*                                                                           */
/*  Practica de Sistemes Informatics en Temps Real                           */
/*                                                                           */
/*  Implementacio d'un sequenciador de MIDI per mitja de Real Time Linux     */
/*  enviar_freq.c: fitxer que amb enviament de frequencies midi i enviament  */
/*                 de la melodia a RT-Linux en el mateix programa.           */
/*                 Versio que imprimeix per pantalla el seguiment de la      */
/*                 execucio i de les comunicacions.                          */
/*                                                                           */
/*  Realitzat per: Francisco Jose Aguilar Celdran i Sergio Blanco Cuaresma   */ 
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <rtl_compat.h>
#include <rtl_fifo.h>
#include "control.h"

char buf[TAM_BUFFER];

/* --------------------------------------------------------------------------- */
/*  lectura_fitxer_melodia: funcio que llegeix el fitxer rebut per la entrada  */
/*                          standard, obtenint els codis midi i les duracions  */
/*                          de tantes entrades com tamany de buffer.           */
/* --------------------------------------------------------------------------- */

int lectura_fitxer_melodia(struct struct_missatge *msg, float segbase) 
{
   int i;
   float duracio_actual;
   int midi_actual;
     
   midi_actual = 1;
   i = 0;
   while ((midi_actual != -1) && (i < TAM_BUFFER)) {
      scanf("%d",&midi_actual);
      scanf("%f",&duracio_actual);
      msg -> midi[i] = midi_actual;
      msg -> tics[i] = (int)(duracio_actual * segbase * RT_TICKS_PER_SEC);
      i++;
   }
   return midi_actual;
}

/* ---------------------------------------------------------------------------------- */
/*  Programa principal: engloba en el mateix programa l'enviament de les frequencies  */
/*                      midi i l'enviament posterior de les notes de la melodia.      */
/*                      llegeix el fitxer de melodia successivament i envia buffers   */
/*                      de notes a RT-Linux fins al final de la melodia.              */
/*                      Es important destacar que es possible la execucio diverses    */
/*                      vegades de melodies sense haver de recarregar el modul de     */
/*                      RT-Linux.                                                     */
/* ---------------------------------------------------------------------------------- */

int main(int argc, char **argv)
{
   int fifo_frequencies;
   int fifo_notes;
   int fifo_entrada;
   int i;
   struct struct_missatge msg;
   struct taula_musical taula_midi;
   float freq;
   FILE *fitxer_midi;
   int midi_actual;
   int fi_melodia;
   char resposta[11];
   float segbase;
   
   if (argc != 2) {
      printf("Us:\n\t %s base_de_temps < fitxer\n", argv[0]);
      return 1;
   } else {
      segbase = atof(argv[1]);
      if (DEBUG) printf("Segbase: %f\n", segbase);
   }

   if (DEBUG) {
      printf("1) Llegim el fitxer de frequencies...\n"); fflush(stdout);
   }   

   i = 0;
   // Lectura del fitxer de frequencies midi
   fitxer_midi = fopen("pitch-freq.txt","r");
   fscanf(fitxer_midi,"%f",&freq);
   while (!feof(fitxer_midi)) {
      //taula_midi.freq[i] = 0.000000;
      //n = freq * 1000;
      taula_midi.freq[i] = freq; //n / 1000;
      // Conversio de frequencies a tics: meitat del periode en segs * RT_TICKS_PER_SEC
      taula_midi.tics_flanc[i] = (1 / (2 * taula_midi.freq[i])) * RT_TICKS_PER_SEC;

      if (DEBUG_NOTES) {
         printf("(%d) %f %d\n",i,taula_midi.freq[i], taula_midi.tics_flanc[i]);
      }
      fscanf(fitxer_midi,"%f",&freq);
      i++;
   }
   fclose(fitxer_midi);
   
   if (DEBUG) {
      printf("2) Obrim les RT-FIFOs...\n"); fflush(stdout);
   }

   // Obrim les RT-FIFOs
   if ((fifo_frequencies = open("/dev/rtf1", O_WRONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf1\n");
      exit(1);
   }
   if ((fifo_notes = open("/dev/rtf2", O_WRONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf2\n");
      exit(1);
   }
   if ((fifo_entrada = open("/dev/rtf3", O_RDONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf3\n");
      exit(1);
   }

   if (DEBUG) {
      printf("3) Enviem frequencies midi entre 50 i 100 a RT-Linux...\n");
      fflush(stdout);
   }
   // Enviem les frequencies midi entre 50 i 100
   if (write(fifo_frequencies,&taula_midi,sizeof(struct taula_musical)) < 0) {
      fprintf(stderr, "Error en enviar les frequencies midi a la RT-Tasca\n");
      exit(1);
   }

   read(fifo_entrada,resposta,9);  // Per comprovar que ha rebut RT-Linux
   if (DEBUG) {
      printf("   Resposta rebuda de RT-LINUX: %s\n",resposta);
   }

   if (DEBUG) {
      printf("4) Lectura primer buffer notes...\n"); fflush(stdout);
   }
 
   // Iniciem la lectura, desde stdin, de la melodia a tocar
   midi_actual = lectura_fitxer_melodia(&msg, segbase);
  
   if (DEBUG_NOTES) {
      for(i=0;i<TAM_BUFFER;i++) {
         printf("---> %d, %d %d\n",i,msg.midi[i],msg.tics[i]); 
      }
   }
   
   if (DEBUG) {
      printf("5) Enviem primer buffer de notes a RT-Linux...\n");
      fflush(stdout);
   }

   // Enviem el primer buffer amb notes i duracions de cada nota
   if (write(fifo_notes,&msg,sizeof(struct struct_missatge)) < 0) {
      fprintf(stderr, "Error en enviar el buffer de notes a la RT-Tasca\n");
      exit(1);
   }
   
   // TEMPORAL:
   //read(fifo_entrada,resposta,9); printf("Resposta rebuda de RT: %s\n",resposta);
   //read(fifo_entrada,&i,sizeof(int)); printf("----> Tics: %d\n",i);
   //read(fifo_entrada,&i,sizeof(int)); printf("----> Midi: %d\n",i);

   // Llegim el fitxer de melodies i enviem el segon buffer de notes
   if (midi_actual != -1) {
      midi_actual = lectura_fitxer_melodia(&msg, segbase);
      if (write(fifo_notes,&msg,sizeof(struct struct_missatge)) < 0) {
         fprintf(stderr, "Error en enviar el buffer de notes a la RT-Tasca\n");
         exit(1);
      }
      if (DEBUG) {
         printf("6) Llegim i enviem segon buffer de notes a RT-Linux...\n");
         fflush(stdout);
      }
   }
   
   if (DEBUG) {
      printf("7) Si !fi_melodia: esperar demanda ");
      printf("notes i enviar a RT-Linux...\n");
      fflush(stdout);
   }

   // Mentre no arribem a la fi de la melodia, esperem peticions de nous buffers de notes
   fi_melodia = 0;
   if (midi_actual == -1) {
      fi_melodia = 1;
   }
   while (!fi_melodia) {
      read(fifo_entrada,resposta,10);
      if (DEBUG) {
         printf("   Nova peticio de buffer rebuda a Linux '%s' ...\n",resposta); 
         fflush(stdout);
      }
      midi_actual = lectura_fitxer_melodia(&msg, segbase);
      if (write(fifo_notes,&msg,sizeof(struct struct_missatge)) < 0) {
         fprintf(stderr, "Error en enviar el buffer de notes a la RT-Tasca\n");
         exit(1);
      }
      if (DEBUG) {
         printf("    Peticio de buffer servida desde Linux...\n"); 
         fflush(stdout);
      }
      if (midi_actual == -1) {
         fi_melodia = 1;
      }
   }
   
   if (DEBUG) {
      printf("Fi de la melodia. Espero que us hagi agradat ;-)\n"); 
      fflush(stdout);
   }

   return 0;
}


